/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jimys.ads.adengine;

import org.drools.FactException;
import org.drools.IntegrationException;
import org.drools.RuleBase;
import org.drools.WorkingMemory;
import org.drools.event.DefaultWorkingMemoryEventListener;
import org.drools.event.ObjectAssertedEvent;
import org.drools.event.ObjectRetractedEvent;
import org.drools.event.WorkingMemoryEventListener;
import org.drools.io.RuleBaseLoader;
import org.drools.rule.Rule;
import org.drools.rule.RuleSet;
import org.drools.spi.Condition;
import org.objectweb.jimys.ads.helper.PumpCheckHelper;
import org.objectweb.jimys.commons.clusterstatus.ClusterStatus;
import org.objectweb.jimys.commons.configuration.Configuration;
import org.objectweb.jimys.commons.configuration.RuleActivator;
import org.objectweb.jimys.commons.exceptions.ProblemException;
import org.objectweb.jimys.commons.probe.AbstractProbe;
import org.objectweb.jimys.commons.rule.RuleHelper;
import org.objectweb.jimys.rar.drools.DroolsConnection;
import org.xml.sax.SAXException;

import javax.ejb.TimerHandle;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.resource.ResourceException;
import javax.resource.cci.ConnectionFactory;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;


/**
 * Wrapper and Singleton to the rule engine
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class Engine {
    /**
     * A handler for the timer
     */
    private TimerHandle th = null;
    /**
     * The drools rulebase reference
     */
    private RuleBase ruleBase = null;
    /**
     * the reference to the drools working memory
     */
    private WorkingMemory workingMemory = null;
    /**
     * Configuration of the engine
     */
    private Configuration config = null;
    /**
     * Table with all possible rule activator. they have been specified in the rules.drl and properties-file
     */
    private Hashtable ruleActivator = null;
    /**
     * counter to know how many events have been received since last fireAllRules
     */
    private int eventCounter = 0;
    /**
     * counter to know how many events have been asserted
     */
    private int assertedObjects = 0;
    /**
     * counter to know how many objects are in the working memory
     */
    private int objectCounter = 0;
    /**
     * to know if drools should write an output if a object is asserted or retracted
     */
    private boolean useListener = false;
    /**
     * the engine configuration
     */
    private Properties props = null;
    /**
     * The engine is a singleton
     */
    private static Engine engine = null;
    /**
     * the connection to the drools resource adapter
     */
    private DroolsConnection con = null;


    private Engine() {
        InputStream rulesFile = Thread.currentThread().getContextClassLoader().getResourceAsStream("rules.drl");
        InputStream configFile = Thread.currentThread().getContextClassLoader().getResourceAsStream("engine.properties");
        props = new Properties();
        try {
            props.load(configFile);
        } catch (IOException e1) {
            e1.printStackTrace();
        }
        String rarname = "";
        if (props.getProperty("engine.rar.connection") != null) {
            rarname = props.getProperty("engine.rar.connection");
        }
        try {
            Context initalContext = new InitialContext();
            ConnectionFactory cf = (ConnectionFactory) initalContext.lookup(rarname);
            con = (DroolsConnection) cf.getConnection();
            System.out.println("con " + con);
        } catch (NamingException e) {
            e.printStackTrace();
        } catch (ResourceException e) {
            e.printStackTrace();
        }
        ruleActivator = new Hashtable();
        System.out.print("Config rule engine");
        this.configEngine();
        System.out.println("...finish ");
        System.out.print("Parse rule file");
        this.parseFile(rulesFile);
        System.out.println("...finish ");
        System.out.print("Init engine");
        this.initEngine();
        System.out.println("...finish ");
    }

    /**
     * get an instance of the engine, which is a singleton
     *
     * @return instance of the engine
     */
    public static synchronized Engine getInstance() {
        if (engine == null) {
            engine = new Engine();
        }
        return engine;
    }

    /**
     * the engine is configurated
     */
    private void configEngine() {
        config = new Configuration();
        //props = new Properties();
        //  props.load(is);
        Enumeration e = props.propertyNames();
        while (e.hasMoreElements()) {
            String propName = (String) e.nextElement();
            if (propName.indexOf("engine.rule.") == -1) {
                String propValue = props.getProperty(propName);
                if (propName.equals("engine.conf.fire.rule.after") && propValue != null) {
                    config.setFireRuleAfter(propValue);
                } else if (propName.equals("engine.conf.fire.interval") && propValue != null) {
                    long value = Long.parseLong(propValue);
                    if (value > 0) {
                        config.setTimeInterval(propValue);
                    } else {
                        config.setTimeInterval("0");
                    }
                } else if (propName.equals("engine.conf.use.listener") && propValue != null) {
                    if (propValue.equals("true")) {
                        useListener = true;
                    }
                } else {
                    if (propValue != null) {
                        config.addValue(propName, propValue);
                    }
                }
            }
        }
    }

    /**
     * loads the rules from the input stream, checks the syntax and creates the rulebase
     *
     * @param is contains the the file with the rules
     */
    private void parseFile(InputStream is) {
        try {
            /*ConflictResolver[] conflictResolvers = new ConflictResolver[]
                    { SalienceConflictResolver.getInstance( ),
                         FifoConflictResolver.getInstance( ) };
            ruleBase = RuleBaseLoader.loadFromInputStream(is,new CompositeConflictResolver( conflictResolvers ));
            */
            ruleBase = RuleBaseLoader.loadFromInputStream(is);
        } catch (IOException e) {
            e.printStackTrace();
        } catch (IntegrationException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        }
    }

    /**
     * does an initialisation work for the engine
     */
    private void initEngine() {
        con.initWorkingMemory(ruleBase);
        workingMemory = (WorkingMemory) con.getWorkingMemory();
        if (useListener) {
            WorkingMemoryEventListener retracted = new DefaultWorkingMemoryEventListener() {
                public void objectRetracted(ObjectRetractedEvent event) {
                    objectCounter--;
                    System.out.println("retracted; obj in mem : " + objectCounter + " ; total asserted : " + assertedObjects);
                }
            };
            workingMemory.addEventListener(retracted);
            WorkingMemoryEventListener asserted = new DefaultWorkingMemoryEventListener() {
                public void objectAsserted(ObjectAssertedEvent event) {
                    objectCounter++;
                    System.out.println("asserted; obj in mem : " + objectCounter + " ; total asserted : " + assertedObjects);
                }
            };
            workingMemory.addEventListener(asserted);
        }
        this.setRuleActivator();
        String clustername = "Jimys";
        if (props.getProperty("clustername") != null) {
            clustername = props.getProperty("clustername");
        }
        workingMemory.setApplicationData("status", new ClusterStatus(clustername));
        workingMemory.setApplicationData("pumps", new Hashtable());
        try {
            objectCounter++;
            assertedObjects++;
            workingMemory.assertObject(new PumpCheckHelper());
            workingMemory.fireAllRules();
        } catch (FactException e) {
            e.printStackTrace();
        }
    }

    /**
     * enables/disables the different rules as specified in the configuration file
     */
    private void setRuleActivator() {
        Enumeration e = this.props.propertyNames();
        while (e.hasMoreElements()) {
            String propName = (String) e.nextElement();
            if (propName.indexOf("engine.rule.") > -1) {
                boolean propValue = Boolean.valueOf(props.getProperty(propName).trim()).booleanValue();
                String ruleSetName = propName.substring((propName.lastIndexOf(".") + 1));
                String ruleset = ruleSetName.substring(0, ruleSetName.indexOf("_"));
                String rulename = ruleSetName.substring((ruleSetName.indexOf("_") + 1));
                this.ruleActivator.put(ruleSetName, new RuleActivator(rulename, ruleset, propValue));
            }
        }
        Collection col = ruleActivator.values();
        Iterator it = col.iterator();
        while (it.hasNext()) {
            RuleActivator r = (RuleActivator) it.next();
            workingMemory.setApplicationData(r.getRuleset() + "_" + r.getRulename(), r);
        }
    }

    /**
     * changes the configuration
     *
     * @param c the new configuration
     */
    public void modifiyConfiguration(Configuration c) {
        if (!c.getFireRuleAfter().equals("")) {
            this.config.setFireRuleAfter(c.getFireRuleAfter());
        }
        if (!c.getTimeInterval().equals("")) {
            this.config.setTimeInterval(c.getTimeInterval());
        }
    }

    /**
     * return the actual rule activators
     *
     * @return a collection with the ruleactivator
     */
    public Collection getRuleActivators() {
        Collection col = new ArrayList();
        Iterator it = ruleActivator.values().iterator();
        while (it.hasNext()) {
            RuleActivator ra = (RuleActivator) it.next();
            col.add(ra);
        }
        return col;
    }

    /**
     * stops the timer
     */
    public void stopTimer() {
        if (th != null) {
            th.getTimer().cancel();
            th = null;
        }
    }

    /**
     * The timer bean needs this to fire all rules
     *
     * @param th the handler to the last timer
     */
    public void fireAllRulesByTimerHandle(TimerHandle th) {
        this.th = th;
        try {
            this.workingMemory.fireAllRules();
        } catch (FactException e) {
            e.printStackTrace();
        }
    }

    /**
     * activate a rule from a ruleset
     *
     * @param ruleset  ruleset name
     * @param rulename name of the rule
     * @throws ProblemException if the rule or ruleset cannot be found
     */
    public void activateRule(String ruleset, String rulename) throws ProblemException {
        if (this.ruleActivator.get(ruleset + "_" + rulename) != null) {
            RuleActivator rs = (RuleActivator) this.ruleActivator.get(ruleset + "_" + rulename);
            rs.setActive(true);
            workingMemory.setApplicationData(ruleset + "_" + rulename, rs);
        } else {
            System.out.println("Cannot activate rule - No such rule found : " + ruleset + "_" + rulename);
            throw new ProblemException("error.jimys.engine.rule.activate.not.possible");
        }
    }


    /**
     * deactivate a rule from a ruleset
     *
     * @param ruleset  ruleset name
     * @param rulename name of the rule
     * @throws ProblemException if the rule or ruleset cannot be found
     */
    public void deactivateRule(String ruleset, String rulename) throws ProblemException {
        if (this.ruleActivator.get(ruleset + "_" + rulename) != null) {
            RuleActivator rs = (RuleActivator) this.ruleActivator.get(ruleset + "_" + rulename);
            rs.setActive(false);
            workingMemory.setApplicationData(ruleset + "_" + rulename, rs);
        } else {
            System.out.println("Cannot deactivate rule - No such rule found : " + ruleset + "_" + rulename);
            throw new ProblemException("error.jimys.engine.rule.deactivate.not.possible");
        }
    }

    /**
     * method to assert an event into the working memory an to fire the rules
     *
     * @param ap an event is type of AbstractProbe
     */
    public synchronized void insertProbe(AbstractProbe ap) {
        try {
            this.workingMemory.assertObject(ap);
            assertedObjects++;
            this.eventCounter++;
            if ((Integer.parseInt(config.getFireRuleAfter())) > 0) {
                if ((Integer.parseInt(config.getFireRuleAfter())) == this.eventCounter) {
                    workingMemory.fireAllRules();
                    this.eventCounter = 0;
                }
            }
        } catch (FactException e) {
            e.printStackTrace();
        }
    }

    /**
     * to clear the agenda of the working memory
     */
    public void clearEngine() {
        workingMemory.clearAgenda();
    }

    /**
     * to get the configuration of the rule engine
     *
     * @return the configuration
     */
    public Configuration getConfig() {
        return config;
    }

    /**
     * get the detail of a rule
     *
     * @param ruleset  the name of the ruleset containing the rule
     * @param rulename the name of the rule
     * @return the detail of the rule
     * @throws ProblemException if the ruleset or rulename cannot be found
     */
    public RuleHelper getRule(String ruleset, String rulename) throws ProblemException {
        List rsl = ruleBase.getRuleSets();
        Iterator it = rsl.iterator();
        while (it.hasNext()) {
            RuleSet rs = (RuleSet) it.next();
            if (rs.getName().equals(ruleset)) {
                if (rs.containsRule(rulename)) {
                    Rule r = rs.getRule(rulename);
                    RuleHelper rh = new RuleHelper();
                    rh.setRulename(r.getName());
                    Iterator rit = r.getConditions().iterator();
                    while (rit.hasNext()) {
                        Condition c = (Condition) rit.next();
                        rh.addCondition(c.toString());
                    }
                    rh.setConsequence(r.getConsequence().toString());
                    return rh;
                } else {
                    throw new ProblemException("error.jimys.engine.no.such.rulename");
                }
            }

        }
        throw new ProblemException("error.jimys.engine.no.such.ruleset");
    }

}
