/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */


package org.objectweb.jimys.ads.adengine.communication;

import org.objectweb.jimys.ads.helper.ResultFactory;
import org.objectweb.jimys.commons.cartography.Cartography;
import org.objectweb.jimys.commons.clusterstatus.ClusterStatus;
import org.objectweb.jimys.commons.probe.AbstractProbe;
import org.objectweb.jimys.dbConnection.valueObjects.ResultValueObject;

import javax.jms.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Serializable;
import java.util.*;

/**
 * connector used to send data to the MessageDriven bean that writes into the database
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class MDBConnector {
    /**
     * the configuration of the engine
     */
    private static Properties props = null;
    /**
     * the topic to send the message to
     */
    private static Topic topic = null;
    /**
     * the topic connection factory to use to get the topic
     */
    private static TopicConnectionFactory topicConnFactory = null;
    /**
     * a collection containing ResultProbeValueObjects
     */
    private Collection results = new ArrayList();
    /**
     * factory to help the creation of ResultValueObject and ResultProbeValueObjects
     */
    private ResultFactory rf = null;

    /**
     * Constructor
     */
    public MDBConnector() {
        if (topicConnFactory == null) {
            props = new Properties();
            try {
                props.load(Thread.currentThread().getContextClassLoader().getResourceAsStream("engine.properties"));
            } catch (FileNotFoundException e1) {
                e1.printStackTrace();
            } catch (IOException e1) {
                e1.printStackTrace();
            }
            String factory = "";
            String url = "";
            if (props.getProperty("database.java.naming.factory.initial") != null) {
                factory = props.getProperty("database.java.naming.factory.initial");
            }

            if (props.getProperty("database.java.naming.provider.url") != null) {
                url = props.getProperty("database.java.naming.provider.url");
            }
            Properties properties = new Properties();
            properties.put("java.naming.factory.initial", factory);
            properties.put("java.naming.provider.url", url);
            try {
                Context ictx = new InitialContext(properties);
                String topicname = "";
                if (props.getProperty("database.topic.name") != null) {
                    topicname = props.getProperty("database.topic.name");
                }
                String topicFactory = "";
                if (props.getProperty("database.topic.connection.factory") != null) {
                    topicFactory = props.getProperty("database.topic.connection.factory");
                }
                topic = (Topic) ictx.lookup(topicname);
                topicConnFactory = (TopicConnectionFactory) ictx.lookup(topicFactory);
                ictx.close();
            } catch (NamingException e) {
                e.printStackTrace();
            }
        }
        rf = new ResultFactory();
    }

    /**
     * adds one attrbute of a probe to the result collection
     *
     * @param ap             the probe
     * @param attributenname the attribute name that is used
     * @param attributeValue the attribute value of that probe
     */
    public void addProbeAttribute(AbstractProbe ap, String attributenname, String attributeValue) {
        results.add(rf.createResultProbeValueObject(ap.getIdentifier(), attributenname, attributeValue));
    }

    /**
     * adds a probe with all its attributes to the result collection
     *
     * @param ap the concerned probe
     */
    public void addProbe(AbstractProbe ap) {
        Collection res = rf.createResultProbeValueObjectCollection(ap.getIdentifier(), ap.getAttributes());
        Iterator it = res.iterator();
        while (it.hasNext()) {
            results.add(it.next());
        }
        res.clear();
    }

    /**
     * sends rhe result to the database message driven bean
     *
     * @param message the message to write in the database
     * @param level   the alarm level
     * @param rule    the rule that has released this consequence
     * @param ruleset the ruleset the rule belongs to
     */
    public void sendResult(String message, String level, String rule, String ruleset) {
        ResultValueObject rvo = rf.createResultValueObject(message, level, rule, ruleset);
        Hashtable send = new Hashtable();
        send.put("resultValueObject", rvo);
        send.put("results", results);
        sendToDB(send);
    }

    /**
     * send the actual clusterstatus to the db message driven bean
     *
     * @param cs the clusterstatus
     */
    public void sendClusterStatus(ClusterStatus cs) {
        sendToDB(cs);
    }

    /**
     * sends the actual cartography to the db message driven bean
     *
     * @param c the cartography
     */
    public void sendCartography(Cartography c) {
        sendToDB(c);
    }

    /**
     * send data to the db message driven bean
     *
     * @param ser the data to send
     */
    public void sendToDB(Serializable ser) {
        try {
            String username = "";
            if (props.getProperty("database.topic.connection.username") != null) {
                username = props.getProperty("database.topic.connection.username");
            }
            String password = "";
            if (props.getProperty("database.topic.connection.password") != null) {
                password = props.getProperty("database.topic.connection.password");
            }
            TopicConnection topicConnection = topicConnFactory.createTopicConnection(username, password);
            topicConnection.start();
            TopicSession topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
            ObjectMessage msg = topicSession.createObjectMessage();
            msg.setObject(ser);
            TopicPublisher pub = topicSession.createPublisher(topic);
            pub.publish(msg);
            pub.close();
            pub = null;
            topicSession.close();
            topicSession = null;
            topicConnection.stop();
            topicConnection.close();
            topicConnection = null;
        } catch (JMSException e) {
            e.printStackTrace();
        }
    }

}

