/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */


package org.objectweb.jimys.ads.helper;

import org.objectweb.jimys.ads.adengine.communication.MDBConnector;
import org.objectweb.jimys.commons.clusterstatus.*;

import java.util.*;

/**
 * helps to check if the data of the different pumps arrive in time and checks if the registered pumps are sending data
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class PumpCheckHelper {
    /**
     * the first timestampe the checker is set to
     */
    private long timestamp = 0;

    /**
     * constructor
     */
    public PumpCheckHelper() {
        Date d = new Date();
        timestamp = d.getTime();
    }

    /**
     * get the last timestamp
     *
     * @return the timestamp
     */
    public long getTimestamp() {
        return timestamp;
    }

    /**
     * sets an new timestamp
     *
     * @param timestamp the new timestamp
     */
    public void setTimestamp(long timestamp) {
        this.timestamp = timestamp;
    }

    /**
     * checks if all the pumps have send their information during the specified interval
     *
     * @param cs       the actual clusterstatus
     * @param pumps    the registered pumps
     * @param interval the time interval to respect
     * @return a new clusterstatus
     */
    public ClusterStatus checkIfAllPumpsInTime(ClusterStatus cs, Hashtable pumps, long interval) {
        if (pumps.size() == 0) {
            System.err.println("No pumps found");
        }
        Enumeration e = pumps.keys();
        while (e.hasMoreElements()) {
            String key = (String) e.nextElement();
            PumpStatus ps = (PumpStatus) pumps.get(key);
            long value = ps.getTimestamp();
            Date d = new Date();
            if ((d.getTime() - value) > interval) {
                ps.setStatus(AbstractStatus.COMPONENT_ERROR);
                cs = setStatus(cs, key, AbstractStatus.COMPONENT_ERROR, d.getTime());
            } else {
                ps.setStatus(AbstractStatus.OK);
            }
        }

        return cs;
    }

    /**
     * sets the clusterstatus to a new status
     *
     * @param cs        the last clusterstatus
     * @param host      the host that has a problem
     * @param error     the error level
     * @param timestamp the timestamp
     * @return new clusterstatus
     */
    private ClusterStatus setStatus(ClusterStatus cs, String host, String error, long timestamp) {
        HostStatus hs = cs.getHostStatus(host);
        boolean updatedb = false;
        if (hs == null) {
            hs = new HostStatus(host, error, timestamp);
            cs.addHostStatus(hs);
            updatedb = true;
        } else {
            if (!hs.getStatus().equals(error)) {
                hs.setStatus(error);
                updatedb = true;
            }
        }
        PumpStatus hsp = hs.getPumpStatus();
        if (hsp == null) {
            hs.setPumpStatus(new PumpStatus(host, error, timestamp));
            updatedb = true;
        } else {
            if (!hsp.getStatus().equals(error)) {
                hsp.setStatus(error);
                updatedb = true;
            }
        }
        Hashtable pumpprobes = hsp.getProbes();
        if (pumpprobes != null) {
            Collection pumpcp = pumpprobes.values();
            Iterator pumppit = pumpcp.iterator();
            while (pumppit.hasNext()) {
                ProbeStatus probe = (ProbeStatus) pumppit.next();
                probe.setStatus(AbstractStatus.UNDEFINED);
            }
        }
        Hashtable domains = hs.getDomains();
        Collection cd = domains.values();
        Iterator dit = cd.iterator();
        while (dit.hasNext()) {
            DomainStatus ds = (DomainStatus) dit.next();
            ds.setStatus(AbstractStatus.UNDEFINED);
            Hashtable instances = ds.getInstances();
            Collection ci = instances.values();
            Iterator iit = ci.iterator();
            while (iit.hasNext()) {
                InstanceStatus is = (InstanceStatus) iit.next();
                is.setStatus(AbstractStatus.UNDEFINED);
                Hashtable probes = is.getProbes();
                Collection cp = probes.values();
                Iterator pit = cp.iterator();
                while (pit.hasNext()) {
                    ProbeStatus probe = (ProbeStatus) pit.next();
                    probe.setStatus(AbstractStatus.UNDEFINED);
                }
            }
        }
        if (updatedb) {
            Date d = new Date();
            cs.setTimestamp(d.getTime());
            cs.setStatus(AbstractStatus.NOK);
            System.out.println("!!!!  PUMPCHECKER : WRITE TO DB");
            MDBConnector mdb = new MDBConnector();
            mdb.sendClusterStatus(cs);
            mdb = null;
        }

        return cs;
    }
}
