/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.dbConnection.session;

import org.objectweb.jimys.dbConnection.valueObjects.ResultValueObject;
import org.jfree.data.xy.XYSeriesCollection;
import org.objectweb.jimys.commons.cartography.Cartography;

import javax.ejb.EJBObject;
import java.rmi.RemoteException;
import java.util.Collection;

/**
 *
 * DBManager
 *
 * @author Georges Goebel, Marc Calvisi
 */
public interface DBManager extends EJBObject {

    /**
     * Adding a result with its accociated values in the Database.
     * @param rvo The result produced by the rule engine
     * @param resultProbeValues The facts which produced the result
     * @throws RemoteException RemoteException
     */
    public void addResult(ResultValueObject rvo, Collection resultProbeValues) throws RemoteException;

    /**
     * Return all the results produced by the rule engine.
     * Attention : If you have a lot of results in the Database, this methode may produce
     * a Transaction rollback (no specific transaction in the DD done)
     * @return The collection of the ResultValueObjects
     * @throws RemoteException
     */
    public Collection findAllResults() throws RemoteException;

    /**
     * Return all the results produced by the rule engine.
     * This methode only returns 20 results and starts with the row number
     * indicated as parameter.
     * ATTENTION : This methode only works with JOnAS and with some Databases
     * which support the rownumber
     * @param numberOfRows The row number to start
     * @return Collection of maximum 20 results
     * @throws RemoteException
     */
    public Collection findAllResults(Integer numberOfRows) throws RemoteException;

    /**
     * Find all the results produced by the rule engine after the indicated time
     * @param time The time to start. This has to be data.getTime()
     * @return Collection of the results (ResultValueObject) after the indicated time
     * @throws RemoteException
     */
    public Collection findAllResultsAfterTime(String time) throws RemoteException;

    /**
     * Find all the results produced by the rule engine which where produced by
     * a specific rule in the specified rule set
     * @param ruleset The ruleset the rule belongs to
     * @param rule The rule
     * @return Collection of ResultValueObjects
     * @throws RemoteException
     */
    public Collection monitorRule(String ruleset, String rule) throws RemoteException;

    /**
     * Find all the results produced by the rule engine which where produced by
     * a specific rule in the specified rule set and after a specified time
     * @param ruleset The ruleset the rule belongs to
     * @param rule The rule
     * @param begintime The time to start
     * @return Collection of ResultValueObjects
     * @throws RemoteException
     */
    public Collection monitorRule(String ruleset, String rule, String begintime) throws RemoteException;

    /**
     * Find all the attribute values for a given probe and a given probe attribute name
     * @param attributeName The probe attribute name
     * @param probeIdentifier The probe identifier
     * @return Collection of ResultProbeValueObject
     * @throws RemoteException
     */
    public Collection findAllAttributes(String attributeName, String probeIdentifier) throws RemoteException;

    /**
     * Find all the attribute values for a given probe and a given probe attribute name
     * after the specified time
     * @param attributeName The probe attribute name
     * @param probeIdentifier The probe identifier
     * @param timestamp The time to start
     * @return Collection of ResultProbeValueObject
     * @throws RemoteException
     */
    public Collection findAllAttributes(String attributeName, String probeIdentifier, String timestamp) throws RemoteException;

    /**
     * Return the list of all the different rules in the Database
     * @return A collection of the rule names (strings)
     * @throws RemoteException
     */
    public Collection findRule() throws RemoteException;

    /**
     * Returns the list of all the probe attributes considered by the specified rule
     * @param rulename The rule
     * @return List of all the probe attribute names (strings)
     * @throws RemoteException
     */
    public Collection findAttributesForARule(String rulename) throws RemoteException;

    /**
     * Find all the different probe identifiers in the system
     * @return List of all the probeidentifiers (strings)
     * @throws RemoteException
     */
    public Collection findAllDistinctProbeIdentifier() throws RemoteException;

    /**
     * Find all the different probe attributes for a specified probe
     * @param probeName The probe identifier
     * @return List of all the attribute names (strings)
     * @throws RemoteException
     */
    public Collection findAllDistinctProbeAttributes(String probeName) throws RemoteException;

    /**
     * Return all the results produced by the rule engine with a specific level
     * For example : all the alarms
     * @param level the level of the result. See LevelHelper
     * @return The list of all the ResultValueObjects
     * @throws RemoteException
     */
    public Collection findAllResultsWithLevel(String level) throws RemoteException;

    /**
     * Return all the results produced by the rule engine with a specific level where
     * a specific probe is conserned
     * For example : all the alarms
     * @param level the level of the result. See LevelHelper
     * @param probeidentifier The probe identifier
     * @return The list of all the ResultValueObjects
     * @throws RemoteException
     */
    public Collection findAllResultsWithLevel(String level, String probeidentifier) throws RemoteException;

    /**
     * Find the number of results produced by the rule engine with a given level for
     * a specified probe
     * @param level The level (See LevelHelper)
     * @param probeidentifier The probe identifier
     * @return Number of results
     * @throws RemoteException
     */
    public Integer findResultsWithLevelCount(String level, String probeidentifier) throws RemoteException;

    /**
     * Add a new cluster status in the database
     * @param clusterStatus the new cluster status
     * @throws RemoteException
     */
    public void addClusterStatus(org.objectweb.jimys.commons.clusterstatus.ClusterStatus clusterStatus) throws RemoteException;

    /**
     * Retrieve the last cluster status from the database
     * @param identifier The cluster identifier
     * @return The last clusterstatus for the idicated cluster
     * @throws RemoteException
     */
    public org.objectweb.jimys.commons.clusterstatus.ClusterStatus getLastClusterStatus(String identifier) throws RemoteException;

    /**
     * Get all the cluster names from the database
     * @return List of all the cluster names
     * @throws RemoteException
     */
    public Collection getClusterNames() throws RemoteException;

    /**
     * Change the level of a result produced by the rule engine
     * Usage : For canceling an alarm
     * @param ResultId The identifier of the result
     * @param level The new level
     * @throws RemoteException
     */
    public void changeLevel(String ResultId, String level) throws RemoteException;

    /**
     * Delete all the data from the database
     * @throws RemoteException
     */
    public void clearAllTables() throws RemoteException;

    /**
     * Add a new cartography in the Database
     * @param carthography The new cartography
     * @throws RemoteException
     */
    public void addCartography(Cartography carthography) throws RemoteException;

    /**
     * Get the cartography for a given hostname
     * @param hostname The hostname
     * @return The cartography for the specified hostname
     * @throws RemoteException
     */
    public Collection getCartography(String hostname) throws RemoteException;

    /**
     * Delete all the data from the Database between the indicated timestamps
     * @param start Start to delete
     * @param end Delete til
     * @throws RemoteException
     */
    public void clearTimePeriod(long start, long end) throws RemoteException;

    /**
     * Load the probevalueobejcts in a resultvalueobject.
     * This is used by showAlarms (in the gui) where we want to optimes the query.
     * Only te resultvalueobject is loaded and not its associated resultprobevalueobjects
     * @param rvo the resultvalueobject to load
     * @return The loaded resultvalueobject
     * @throws RemoteException
     */
    public ResultValueObject loadProbes (ResultValueObject rvo) throws RemoteException;

    /**
     * Retrieve a JFreechart object which is filled with all the data. This enables to
     * gain performance.
     * @param ruleset The name of the ruleset
     * @param rule The rule name
     * @param attribute The probe attribute to monitor
     * @param begintime The begin time
     * @return The jfreechart object
     * @throws RemoteException
     */
    public XYSeriesCollection getMonitorRuleChart (String ruleset, String rule, String attribute, String begintime) throws RemoteException;

    /**
     * Find all the probes which match a query pattern and have a specific level
     * @param query The query pattern. A '%'is append to the query
     * @param level The level of the results
     * @return The collection of the resultprobes which match the request.
     * @throws RemoteException
     */
    public Collection findProbeWithStatus(String query, String level) throws RemoteException;

    /**
     * Cancel all the results in the database which have the level given as parameter
     * @param level The level of the results which are canceled
     * @throws RemoteException
     */
    public void cancelAllAlarms (String level) throws RemoteException;
}
