/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id:  $
* --------------------------------------------------------------------------
*/


package org.objectweb.jimys.commons.clusterstatus;

import java.io.Serializable;
import java.util.Date;

/**
*
*  abstract status
*
* @author Georges Goebel, Marc Calvisi
*/
public abstract class AbstractStatus implements Serializable {
    /**
     * value if the status has no problems
     */
    public static final String OK = "ok";
    /**
     * status if a component or the cluster has a problem
     */
    public static final String NOK = "nok";
    /**
     * value of a probe if it detects that the component it is monitoring does not work
     */
    public static final String COMPONENT_ERROR = "component error";
    /**
     * value of a probe of the pump detects that the probe is not working proberly
     */
    public static final String PROBE_ERROR = "probe error";
    /**
     * status of a pump if it has not been checked by the PumpChecker
     */
    public static final String NOT_CHECKED = "not checked";
    /**
     * value if the status is not known.
     */
    public static final String UNDEFINED = "undefined";

    /**
     * html-color-code of the status ok
     */
    private static final String COLOR_OK = "#008000";

    /**
     * html-color-code of the status  nok
     */
    private static final String COLOR_NOK = "#FF0000";
    /**
     * html-color-code of the status component error
     */
    private static final String COLOR_COMPONENT_ERROR = "#FF0000";
    /**
     * html-color-code of the status probe error
     */
    private static final String COLOR_PROBE_ERROR = "#FF0000";
    /**
     * html-color-code of the status not checked
     */
    private static final String COLOR_NOT_CHECKED = "#FFA500";
    /**
     * html-color-code of the status undefined
     */
    private static final String COLOR_UNDEFINED = "#FFA500";
    /**
     * identifier of the cluster
     */
    private String identifier = "";
    /**
     * actual global status of the cluster
     */
    private String status = "";
    /**
     * timestamp of the current status
     */
    private long timestamp = -1;

    /**
     * Constructor
     * @param identifier identifier of the cluster
     * @param status actual status of the cluster
     * @param timestamp timestamp of the status
     */
    protected AbstractStatus(String identifier, String status, long timestamp) {
        this.identifier = identifier;
        this.status = status;
        this.timestamp = timestamp;
    }

    /**
     * returns the identifier of the cluster
     * @return the cluster identifier
     */
    public String getIdentifier() {
        return identifier;
    }

    /**
     * set the identifier
     * @param identifier the new identifier of the cluster
     */
    public void setIdentifier(String identifier) {
        this.identifier = identifier;
    }

    /**
     * return the actual status of the cluster
     * @return the status
     */
    public String getStatus() {
        return status;
    }

    /**
     * set a new status
     * @param status the actual new status
     */
    public void setStatus(String status) {
        this.status = status;
    }

    /**
     * returns the timestamp of the cluster status
     * @return the timestamp of the cluster status
     */
    public long getTimestamp() {
        return timestamp;
    }

    /**
     * set the timestamp of the cluster status to a new value
     * @param timestamp the new timestamp
     */
    public void setTimestamp(long timestamp) {
        this.timestamp = timestamp;
    }

    /**
     * returns a formatted timestamp of the cluster
     * @return the actual timestamp converted as Date
     */
    public Date getTime() {
        return new Date(getTimestamp());
    }

    /**
     * returns the component name and its status
     * @return name and status of a component
     */
    public abstract String getNameAndStatus();

    /**
     * return the html-color for the cluster status
     * @return the html-color for the actual status
     */
    protected String getHtmlStatus() {
        String formatColor = COLOR_NOT_CHECKED;
        if (getStatus().equals(OK)) {
            formatColor = COLOR_OK;
        } else {
            if (getStatus().equals(NOK)) {
                formatColor = COLOR_NOK;
            } else {
                if (getStatus().equals(COMPONENT_ERROR)) {
                    formatColor = COLOR_COMPONENT_ERROR;
                } else {
                    if (getStatus().equals(PROBE_ERROR)) {
                        formatColor = COLOR_PROBE_ERROR;
                    } else {
                        if (getStatus().equals(COLOR_UNDEFINED)) {
                            formatColor = COLOR_UNDEFINED;
                        }
                    }
                }
            }
        }

        String result = "<font color=" + formatColor + ">" + getStatus() + "</font>";
        return result;
    }

}
