/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.ads.mediator;

import org.objectweb.jimys.ads.mediator.communication.PumpAdr;
import org.objectweb.jimys.ads.mediator.communication.PumpConnector;
import org.objectweb.jimys.ads.mediator.communication.PumpConnectorHome;
import org.objectweb.jimys.ads.mediator.communication.EngineConnector;
import org.objectweb.jimys.ads.mediator.parser.ParseXMLBean;
import org.objectweb.jimys.commons.exceptions.ProblemException;
import org.objectweb.jimys.commons.probe.Admin;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;

/**
 * Mediator Singleton
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class Mediator {
    /**
     * the pumps the mediator is connected to
     */
    private static Hashtable connectedPumps = new Hashtable();    /*hostname,pumpAdr*/
    /**
     * the session with the connection to the pumps
     */
    private static Hashtable sessions = new Hashtable();           /*hostname,pumpAdr*/
    /**
     * the pumps that are available to connect to
     */
    private static Hashtable availablePumps = new Hashtable();    /*hostname,pumpAdr*/
    /**
     * the configuration of the mediator
     */
    private static Properties props = new Properties();
    /**
     * the mediator instance
     */
    private static Mediator mediator = null;

    /**
     * constructor
     */
    public Mediator() {
        InputStream is = Thread.currentThread().getContextClassLoader().getResourceAsStream("mediator.properties");
        try {
            props.load(is);
            extractAvailablePumps();
        } catch (IOException e1) {
            e1.printStackTrace();
        }
    }

    /**
     * get an instance of the mediator
     * @return the mediator instance
     */
    public static Mediator getInstance() {
        if (mediator == null) {
            mediator = new Mediator();
        }
        return mediator;
    }

    /**
     * read the available pumps from the configuration file
     */
    private void extractAvailablePumps() {
        synchronized (availablePumps) {
            Enumeration e = this.props.propertyNames();
            while (e.hasMoreElements()) {
                String propName = (String) e.nextElement();
                if (propName.indexOf("mediator.pump.host.") > -1) {
                    String adr = props.getProperty(propName).trim();
                    StringTokenizer st = new StringTokenizer(adr, ":");
                    PumpAdr pa = new PumpAdr(st.nextToken(), Integer.parseInt(st.nextToken()));
                    availablePumps.put(pa.getHostname(), pa);
                }
            }
        }
    }

    /**
     * connect to a list of a pump
     * @param hosts he list of the pumps to connect to
     * @throws ProblemException if a problem exists
     */
    public void connectToPumps(String[] hosts) throws ProblemException{
        synchronized (availablePumps) {
            ProblemException pe = null;
            for (int i = 0; i < hosts.length; i++) {
                PumpAdr pa = (PumpAdr) availablePumps.get(hosts[i]);
                try {
                    connectToPump(pa.getHostname(), pa.getPort());
                } catch (ProblemException e) {
                    e.printStackTrace();
                    pe = e;
                }
            }
            if (pe!=null) throw pe;
        }
    }

    /**
     * connect to a pump
     * @param host the host of the pump
     * @param port the port on that host
     * @throws ProblemException if a problem exists
     */
    public void connectToPump(String host, int port) throws ProblemException {
        synchronized (connectedPumps) {
            if (connectedPumps.containsKey(host)) {
                System.out.println("Cannot connect a 2nd time to a pump");
                return;
            }
            PumpAdr pump = new PumpAdr(host, port);
            System.out.println("Connect to pump : " + pump.getHostname());
            PumpConnector pc = null;
            try {
                Context initialContext = new InitialContext();
                String ejbname = "";
                if (props.getProperty("mediator.pump.connector.local.home") != null) {
                    ejbname = props.getProperty("mediator.pump.connector.local.home");
                }
                PumpConnectorHome phome = (PumpConnectorHome) initialContext.lookup(ejbname);
                pc = phome.create();
                pc.run(pump.getHostname(), Integer.toString(pump.getPort()));
                synchronized (sessions) {
                    sessions.put(host, pc);
                }
                connectedPumps.put(host, pump);
            } catch (NamingException e) {
                e.printStackTrace();
                if (pc != null) {
                    try {
                        pc.remove();
                    } catch (RemoveException re) {
                        re.printStackTrace();
                    }
                }
            } catch (CreateException e) {
                e.printStackTrace();
                if (pc != null) {
                    try {
                        pc.remove();
                    } catch (RemoveException re) {
                        re.printStackTrace();
                    }
                }
            } catch (ProblemException e) {
                e.printStackTrace();
                if (pc != null) {
                    try {
                        pc.remove();
                    } catch (RemoveException re) {
                        re.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * disconnect from a pump
     * @param host the host with the pump
     * @throws ProblemException
     */
    public void disconnectFromPump(String host) throws ProblemException {
        synchronized (connectedPumps) {
            PumpAdr pump = (PumpAdr) connectedPumps.get(host);
            if (pump == null) {
                System.out.println("host : <" + host + "> unknown");
                throw new ProblemException("host : <" + host + "> unknown");
            }
            System.out.println("Disconnect from pump : " + pump.getHostname());
            try {
                PumpConnector pc = (PumpConnector) sessions.get(host);
                pc.stopConnection();
                pc.remove();
                connectedPumps.remove(host);
                synchronized (sessions) {
                    sessions.remove(host);
                }
                EngineConnector ec = new EngineConnector();
                Admin admin = new Admin((new Date()).getTime(),host+";domain;instance;admin;CONTROL",null);
                admin.setRemoveHost(true);
                ec.sendProbeToRuleEngine(admin);
            } catch (RemoveException e) {
                e.printStackTrace();
            } catch (ProblemException e){
                e.printStackTrace();
            }
            connectedPumps.remove(host);
            synchronized (sessions) {
                sessions.remove(host);
            }
            EngineConnector ec = new EngineConnector();
            Admin admin = new Admin((new Date()).getTime(),host+";domain;instance;admin;CONTROL",null);
            admin.setRemoveHost(true);
            ec.sendProbeToRuleEngine(admin);
        }
    }

    /**
     * disconnect from all pumps the mediator is connected to
     * @throws ProblemException if a problem exists
     */
    public void disconnectFromPumps()throws ProblemException {
        Collection pumps = new Vector(connectedPumps.values());
        Iterator it = pumps.iterator();
        ProblemException pe = null;
        while (it.hasNext()) {
            try {
                disconnectFromPump(((PumpAdr) it.next()).getHostname());
            } catch (ProblemException e) {
                e.printStackTrace();
                pe = e;
            }
        }
        if (pe!=null) throw pe;
    }

    /**
     * disconnect from a list of hosts
     * @param hosts all the hosts to disconnect from
     * @throws ProblemException if a problem exists
     */
    public void disconnectFromPumps(String[] hosts) throws ProblemException {
        ProblemException pe = null;
        for (int i = 0; i < hosts.length; i++) {
            try {
                disconnectFromPump(hosts[i]);
            } catch (ProblemException e) {
                e.printStackTrace();
                pe =e;
            }
        }
        if (pe!=null) throw pe;
    }

    /**
     * the the collection with all the hostname of the connected pumps
     * @return all hostnames
     */
    public Collection getConnectedPumps() {
        synchronized (connectedPumps) {
            Collection col = new ArrayList();
            Enumeration e = connectedPumps.keys();
            while (e.hasMoreElements()) {
                col.add(e.nextElement());
            }
            return col;
        }
    }

    /**
     * the the collection with all the hostname of available pumps
     * @return all hostnames
     */
    public Collection getAvailablePumps() {
        synchronized (availablePumps) {
            Collection col = new ArrayList();
            Enumeration e = availablePumps.keys();
            while (e.hasMoreElements()) {
                String adr = (String) e.nextElement();
                synchronized (connectedPumps) {
                    if (!connectedPumps.containsKey(adr)) {
                        col.add(adr);
                    }
                }
            }
            return col;
        }
    }

    /**
     * helps to check the status of all connected pumps
     * @return status of the different pumps
     * @throws EJBException
     */
    public Collection checkCluster() throws EJBException {
        synchronized (sessions) {
            Collection col = new ArrayList();
            Iterator it = sessions.values().iterator();
            while (it.hasNext()) {
                PumpConnector pc = (PumpConnector) it.next();
                try {
                    col.add(pc.checkNode());
                } catch (ProblemException e) {
                    e.printStackTrace();
                }

            }
            return col;
        }
    }

    /**
     * get a list of the probe that are used by a pump
     * @param hostname the hostname of the pump to check
     * @return the list with all probes
     * @throws ProblemException
     */
    public Collection getRegisteredProbesFromPump(String hostname) throws ProblemException {
        synchronized (sessions) {
            if (!sessions.containsKey(hostname)) {
                throw new ProblemException("host : <" + hostname + "> unknown");
            }
            PumpConnector pc = (PumpConnector) sessions.get(hostname);
            return pc.getRegisteredProbesFromPump();
        }
    }

    /**
     * get the sample rate of a probe from a pump on a host
     * @param hostname the host to connect to
     * @param probename the probe name
     * @return the actual sample rate
     * @throws ProblemException
     */
    public Long getProbeSampleRate(String hostname, String probename) throws ProblemException {
        synchronized (sessions) {
            if (!sessions.containsKey(hostname)) {
                throw new ProblemException("host : <" + hostname + "> unknown");
            }
            PumpConnector pc = (PumpConnector) sessions.get(hostname);
            return pc.getProbeSampleRate(probename);
        }
    }

    /**
     * set the sample rate of a probe from a pump on a host
     * @param hostname the host to connect to
     * @param probename the probe name
     * @param sampleRate the new value for the sample rate
     * @throws ProblemException
     */
    public void setProbeSampleRate(String hostname, String probename, Long sampleRate) throws ProblemException {
        synchronized (sessions) {
            if (!sessions.containsKey(hostname)) {
                throw new ProblemException("host : <" + hostname + "> unknown");
            }
            PumpConnector pc = (PumpConnector) sessions.get(hostname);
            pc.setProbeSampleRate(probename, sampleRate);
        }
    }

    /**
     * get the jmx name of the pump
     * @return the name of the pump
     * @throws ProblemException
     */
    public String getPumpName() throws ProblemException {
        if (props.getProperty("mediator.connect.pump") != null) {
            return props.getProperty("mediator.connect.pump");
        }
        return "";
    }

    /**
     * connect to all available pumps
     * @throws ProblemException if a problem exists
     */
    public void connectToAll() throws ProblemException{
        synchronized (availablePumps) {
            Enumeration e = availablePumps.elements();
            ProblemException pe = null;
            while (e.hasMoreElements()) {
                PumpAdr pa = (PumpAdr) e.nextElement();
                if (!connectedPumps.containsKey(pa.getHostname())) {
                    try {
                        connectToPump(pa.getHostname(), pa.getPort());
                    } catch (ProblemException ee) {
                        ee.printStackTrace();
                        pe = ee;
                    }
                }
            }
            if (pe!=null) throw pe;
        }
    }

    /**
     * get the configuration of the mediator
     * @return the properties
     */
    public Properties getConfiguration() {
        return props;
    }
}
