/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.ads.mediator.parser;

import org.objectweb.jimys.commons.cartography.*;
import org.objectweb.jimys.commons.probe.AbstractProbe;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;

/**
 *  cartography parser
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class CartographyProbeParser extends AbstractProbeParser {
    private String identifier = "";
    private String timeStamp = "";

    /**
     * constructor
     * @param node the xml node containing this probe
     * @param id the id of the probe
     */
    public CartographyProbeParser(Node node, String id) {
        super(node, id);
    }

    /**
     * get the probe object
     * @return probe object
     */
    public AbstractProbe getAbstractProbe() {
        identifier = ((Element) getNode()).getAttribute("id");
        timeStamp = ((Element) getNode()).getAttribute("time");
        return new Cartography(Long.parseLong(timeStamp), identifier, getAttributes(getNode()));
    }

    /**
     * split the node into the different attributes
     * @param node the node with the attributes
     * @return all attributes
     */
    public Hashtable getAttributes(Node node) {
        Hashtable attr = new Hashtable();
        NodeList nl = node.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            Node n = nl.item(i);
            String name = ((Element) n).getAttribute("probeClass");
            if (name.equals("org.objectweb.lewys.cartography.linux.Probe_Controller")) {
                attr.put("Controller", getController(n));
            } else if (name.equals("org.objectweb.lewys.cartography.linux.Probe_DiskPartition")) {
                attr.put("DiskPartition", getDiskPartition(n));
            } else if (name.equals("org.objectweb.lewys.cartography.linux.Probe_IPPort")) {
                attr.put("IPPort", getIPPort(n));
            } else if (name.equals("org.objectweb.lewys.cartography.linux.Probe_Memory")) {
                attr.put("Memory", getMemory(n));
            } else if (name.equals("org.objectweb.lewys.cartography.linux.Probe_OperatingSystem")) {
                attr.put("OperatingSystem", getOperatingSystem(n));
            }
        }
        return attr;
    }

    /**
     * get the attribute of the nodes
     * @param start start from
     * @param length length of the attribute
     * @param nl list with all nodes
     * @return a list with all attributes
     */
    private Hashtable getNodeAttributes(int start, int length, NodeList nl) {
        Hashtable attr = new Hashtable();
        for (int y = start; y < start + length; y++) {
            Element e = (Element) nl.item(y);
            String strName = e.getAttribute("name");
            NodeList childValue = e.getChildNodes();
            if (childValue.getLength() > 0) {
                if (childValue.item(0).getNodeType() == Node.TEXT_NODE) {
                    String value = childValue.item(0).getNodeValue();
                    attr.put(strName, value);
                }
            } else {
                attr.put(strName, "no_value");
            }
        }
        return attr;
    }

    /**
     * get the controller cartography
     * @param node the node with the controller
     * @return a collection with all controllers
     */
    private Collection getController(Node node) {
        Collection col = new ArrayList();
        NodeList nl = node.getChildNodes();
        int i = 0;
        while (i < nl.getLength()) {
            Hashtable attr = getNodeAttributes(i, 3, nl);
            CartoController c = new CartoController(Long.parseLong(this.timeStamp), this.identifier + "_Controller_" + ((String) attr.get("Name")));
            c.setAttributes(attr);
            col.add(c);
            i = i + 3;
        }
        return col;
    }
    /**
     * get the disk partition cartography
     * @param node the node with the disk partitions
     * @return a collection with all disk partitions
     */
    private Collection getDiskPartition(Node node) {
        Collection col = new ArrayList();
        NodeList nl = node.getChildNodes();
        int i = 0;
        while (i < nl.getLength()) {
            Hashtable attr = getNodeAttributes(i, 5, nl);
            CartoDiskPartition dp = new CartoDiskPartition(Long.parseLong(this.timeStamp), this.identifier + "_DiskPartition_" + ((String) attr.get("Name")));
            dp.setAttributes(attr);
            col.add(dp);
            i = i + 5;
        }
        return col;
    }
    /**
     * get the IPPort cartography
     * @param node the node with the IPPort
     * @return a collection with all IPPort
     */
    private Collection getIPPort(Node node) {
        Collection col = new ArrayList();
        NodeList nl = node.getChildNodes();
        int i = 0;
        while (i < nl.getLength()) {
            Hashtable attr = getNodeAttributes(i, 4, nl);
            CartoIPPort ipp = new CartoIPPort(Long.parseLong(this.timeStamp), this.identifier + "_IPPort_" + ((String) attr.get("InterfaceName")));
            ipp.setAttributes(attr);
            col.add(ipp);
            i = i + 4;
        }
        return col;
    }
    /**
     * get the memory cartography
     * @param node the node with the memory
     * @return a collection with all memory
     */
    private Collection getMemory(Node node) {
        Collection col = new ArrayList();
        NodeList nl = node.getChildNodes();
        int i = 0;
        while (i < nl.getLength()) {
            Hashtable attr = getNodeAttributes(i, 2, nl);
            CartoMemory m = new CartoMemory(Long.parseLong(this.timeStamp), this.identifier + "_Memory_" + ((String) attr.get("MemoryType")));
            m.setAttributes(attr);
            col.add(m);
            i = i + 2;
        }
        return col;
    }
    /**
     * get the operating system cartography
     * @param node the node with the operating system
     * @return a collection with all operating systems
     */
    private Collection getOperatingSystem(Node node) {
        Collection col = new ArrayList();
        NodeList nl = node.getChildNodes();
        int i = 0;
        while (i < nl.getLength()) {
            Hashtable attr = getNodeAttributes(i, 6, nl);
            CartoOperatingSystem os = new CartoOperatingSystem(Long.parseLong(this.timeStamp), this.identifier + "_OperatingSystem_" + ((String) attr.get("Name ")));
            os.setAttributes(attr);
            col.add(os);
            i = i + 6;
        }
        return col;
    }
    /**
     * get the processor cartography
     * @param node the node with the processors
     * @return a collection with all processors
     */
    private Collection getProcessor(Node node) {
        Collection col = new ArrayList();
        NodeList nl = node.getChildNodes();
        int i = 0;
        while (i < nl.getLength()) {
            Hashtable attr = getNodeAttributes(i, 6, nl);
            CartoProcessor p = new CartoProcessor(Long.parseLong(this.timeStamp), this.identifier + "_Processor_" + ((String) attr.get("ProcessorID")));
            p.setAttributes(attr);
            col.add(p);
            i = i + 6;
        }
        return col;
    }
}
