/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.admin.security;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.objectweb.jonas.webapp.admin.mbean.MBeanUtils;

/**
 * @author Michel-Ange ANTON
 */
public class SecurityMBeanUtils {

// ----------------------------------------------------------- Constants

// --------------------------------------------------------- Public Methods
    /**
     * Return the list of Security Memory Factories in the current server.
     *
     * @return The list of Security Memory Factories
     * @throws ManagementException
     * @throws MalformedObjectNameException
     */
    public static ArrayList getSecurityMemoryFactories(MBeanServer p_MBeanServer)
        throws Exception {
        String sName;
        ArrayList al = new ArrayList();
        Iterator itNames = MBeanUtils.getListMBeanName(p_MBeanServer
            , JonasObjectName.allSecurityMemoryFactories()).iterator();
        while (itNames.hasNext()) {
            sName = MBeanUtils.extractValueMbeanName(",name", itNames.next().toString());
            if (sName != null) {
                al.add(sName);
            }
        }
        Collections.sort(al);
        return al;
    }

    /**
     * Return the list of Security Datasource Factories in the current server.
     *
     * @return The list of Security Datasource Factories
     * @throws ManagementException
     * @throws MalformedObjectNameException
     */
    public static ArrayList getSecurityDatasourceFactories(MBeanServer p_MBeanServer)
        throws Exception {
        String sName;
        ArrayList al = new ArrayList();
        Iterator itNames = MBeanUtils.getListMBeanName(p_MBeanServer
            , JonasObjectName.allSecurityDatasourceFactories()).iterator();
        while (itNames.hasNext()) {
            sName = MBeanUtils.extractValueMbeanName(",name", itNames.next().toString());
            if (sName != null) {
                al.add(sName);
            }
        }
        Collections.sort(al);
        return al;
    }

    /**
     * Return the list of Security Ldap Factories in the current server.
     *
     * @return The list of Security Ldap Factories
     * @throws ManagementException
     * @throws MalformedObjectNameException
     */
    public static ArrayList getSecurityLdapFactories(MBeanServer p_MBeanServer)
        throws Exception {
        String sName;
        ArrayList al = new ArrayList();
        Iterator itNames = MBeanUtils.getListMBeanName(p_MBeanServer
            , JonasObjectName.allSecurityLdapFactories()).iterator();
        while (itNames.hasNext()) {
            sName = MBeanUtils.extractValueMbeanName(",name", itNames.next().toString());
            if (sName != null) {
                al.add(sName);
            }
        }
        Collections.sort(al);
        return al;
    }

    /**
     * Return the SubType of a Security Factory in the current server.
     *
     * @param p_NameFactory The factory name to find
     * @return The SubType or null if not found
     * @throws ManagementException
     * @throws MalformedObjectNameException
     */
    public static String findSecurityFactorySubType(MBeanServer p_MBeanServer, String p_NameFactory)
        throws Exception {
        String sName;
        String sSubType = null;
        ObjectName on = null;

        Iterator itNames = MBeanUtils.getListMBeanName(p_MBeanServer
            , JonasObjectName.allSecurityFactories()).iterator();
        while (itNames.hasNext()) {
            on = new ObjectName(itNames.next().toString());
            sName = on.getKeyProperty("name");
            if (p_NameFactory.equals(sName) == true) {
                sSubType = on.getKeyProperty("subtype");
                break;
            }
        }
        return sSubType;
    }

    /**
     * Return the list of all users in a resource.
     *
     * @param p_Resource The resource
     * @return The list of users
     * @throws ManagementException
     * @throws MalformedObjectNameException
     */
    public static ArrayList getUsers(MBeanServer p_MBeanServer, String p_Resource)
        throws Exception {
        String sName;
        ArrayList al = new ArrayList();
        Iterator itNames = MBeanUtils.getListMBeanName(p_MBeanServer
            , JonasObjectName.allUsers(p_Resource)).iterator();
        while (itNames.hasNext()) {
            sName = MBeanUtils.extractValueMbeanName(",name", itNames.next().toString());
            if (sName != null) {
                al.add(sName);
            }
        }
        Collections.sort(al);
        return al;
    }

    /**
     * Return the list of all roles in a resource.
     *
     * @param p_Resource The resource
     * @return The list of roles
     * @throws ManagementException
     * @throws MalformedObjectNameException
     */
    public static ArrayList getRoles(MBeanServer p_MBeanServer, String p_Resource)
        throws Exception {
        String sName;
        ArrayList al = new ArrayList();
        Iterator itNames = MBeanUtils.getListMBeanName(p_MBeanServer
            , JonasObjectName.allRoles(p_Resource)).iterator();
        while (itNames.hasNext()) {
            sName = MBeanUtils.extractValueMbeanName(",name", itNames.next().toString());
            if (sName != null) {
                al.add(sName);
            }
        }
        Collections.sort(al);
        return al;
    }

    /**
     * Return the list of all groups in a resource.
     *
     * @param p_Resource The resource
     * @return The list of groups
     * @throws ManagementException
     * @throws MalformedObjectNameException
     */
    public static ArrayList getGroups(MBeanServer p_MBeanServer, String p_Resource)
        throws Exception {
        String sName;
        ArrayList al = new ArrayList();
        Iterator itNames = MBeanUtils.getListMBeanName(p_MBeanServer
            , JonasObjectName.allGroups(p_Resource)).iterator();
        while (itNames.hasNext()) {
            sName = MBeanUtils.extractValueMbeanName(",name", itNames.next().toString());
            if (sName != null) {
                al.add(sName);
            }
        }
        Collections.sort(al);
        return al;
    }

}
