/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.taglib;

import java.io.Serializable;
import java.util.HashMap;

/**
 * <p>The overall data structure representing a <em>tree control</em>
 * that can be rendered by the <code>TreeControlTag</code> custom tag.
 * Each node of the tree is represented by an instance of
 * <code>TreeControlNode</code>.</p>
 *
 * @author Jazmin Jonson
 * @author Craig R. McClanahan
 * @version $Revision: 1.4
 */

public class TreeControl implements Serializable {
    public static final String ID_PREFIX = "treenode";

// ----------------------------------------------------------- Constructors

    /**
     * Construct a new instance with no predefined root node.
     */
    public TreeControl() {
        super();
        setRoot(null);
    }

    /**
     * Construct a new instance with the specified root node.
     *
     * @param root The new root node
     */
    public TreeControl(TreeControlNode root) {
        super();
        setRoot(root);
    }

// ----------------------------------------------------- Instance Variables

    /**
     * The collection of nodes that represent this tree, keyed by name.
     */
    protected HashMap registry = new HashMap();

    /**
     * The most recently selected node.
     */
    protected TreeControlNode selected = null;

    // Id
    protected int mi_Id = 0;

// ------------------------------------------------------------- Properties

    /**
     * The root node of the entire tree.
     */
    protected TreeControlNode root = null;

    public TreeControlNode getRoot() {
        return (this.root);
    }

    protected void setRoot(TreeControlNode root) {
        if (this.root != null) {
            removeNode(this.root);
        }
        if (root != null) {
            addNode(root);
        }
        root.setLast(true);
        this.root = root;
    }

    /**
     * The current displayable "width" of this tree (that is, the maximum
     * depth of the visible part of the tree).
     */
    public int getWidth() {
        if (root == null) {
            return (0);
        }
        else {
            return (getWidth(root));
        }
    }

// --------------------------------------------------------- Public Methods

    /**
     * Find and return the <code>TreeControlNode</code> for the specified
     * node name, if it exists; otherwise, return <code>null</code>.
     *
     * @param name Name of the <code>TreeControlNode</code> to be returned
     */
    public TreeControlNode findNode(String name) {
        synchronized (registry) {
            return ((TreeControlNode) registry.get(name));
        }
    }

    /**
     * Mark the specified node as the one-and-only currently selected one,
     * deselecting any previous node that was so marked.
     *
     * @param node Name of the node to mark as selected, or <code>null</code>
     *  if there should be no currently selected node
     */
    public void selectNode(String name) {
        if (selected != null) {
            selected.setSelected(false);
            selected = null;
        }
        selected = findNode(name);
        if (selected != null) {
            selected.setSelected(true);
        }
    }

    /**
     * Get the last node selected.
     *
     * @return the current node selected
     */
    public TreeControlNode getSelected() {
        return selected;
    }

    /**
     * Expand a branch in the tree of the selected node.
     */
    public void expandSelectedParents() {
        TreeControlNode oCur = getSelected();
        while (oCur != null) {
            oCur = oCur.getParent();
            if (oCur != null) {
                oCur.setExpanded(true);
            }
        }
    }

    public String newId() {
        StringBuffer sbRet = new StringBuffer(ID_PREFIX);
        sbRet.append(mi_Id);
        mi_Id++;
        return sbRet.toString();
    }

// -------------------------------------------------------- Package Methods

    /**
     * Register the specified node in our registry of the complete tree.
     *
     * @param node The <code>TreeControlNode</code> to be registered
     *
     * @exception IllegalArgumentException if the name of this node
     *  is not unique
     */
    void addNode(TreeControlNode node)
        throws IllegalArgumentException {
        synchronized (registry) {
            String name = node.getName();
            if (registry.containsKey(name)) {
                throw new IllegalArgumentException("Name '" + name + "' is not unique");
            }
            node.setTree(this);
            registry.put(name, node);
            // Refresh expand info
            autoRefresh(node);
        }
    }

    /**
     * Calculate the width of the subtree below the specified node.
     *
     * @param node The node for which to calculate the width
     */
    int getWidth(TreeControlNode node) {
        int width = node.getWidth();
        if (!node.isExpanded()) {
            return (width);
        }
        TreeControlNode children[] = node.findChildren();
        for (int i = 0; i < children.length; i++) {
            int current = getWidth(children[i]);
            if (current > width) {
                width = current;
            }
        }
        return (width);
    }

    /**
     * Deregister the specified node, as well as all child nodes of this
     * node, from our registry of the complete tree.  If this node is not
     * present, no action is taken.
     *
     * @param node The <code>TreeControlNode</code> to be deregistered
     */
    void removeNode(TreeControlNode node) {
        synchronized (registry) {
            TreeControlNode children[] = node.findChildren();
            for (int i = 0; i < children.length; i++) {
                removeNode(children[i]);
            }
            TreeControlNode parent = node.getParent();
            if (parent != null) {
                parent.removeChild(node);
            }
            node.setParent(null);
            node.setTree(null);
            if (node == this.root) {
                this.root = null;
            }
            registry.remove(node.getName());
            // Save removed node in list
            addRemovedList(node);
        }
    }

    /**
     * List to save removed node used by the auto-refresh mode.
     *
     */
    private HashMap m_RemovedList = null;

    /**
     * Disable auto-refresh mode.
     */
    public void disableAutoRefresh() {
        if (m_RemovedList != null) {
            m_RemovedList.clear();
        }
        m_RemovedList = null;
    }

    /**
     * Enable auto-refresh mode.
     * When a set of children are refreshed (removed then added),
     * the expanded info is copied of the removed node to the added node.
     * The name is used to retreive the good node.
     * By default, the auto-refresh mode is disabled.
     * Be careful, enabled this mode before the remove of all nodes and
     * disabled it after the add.
     */
    public void enableAutoRefresh() {
        m_RemovedList = new HashMap();
    }

    /**
     * Add the removed node in the removed list.
     *
     * @param p_RemovedNode The removed node
     */
    void addRemovedList(TreeControlNode p_RemovedNode) {
        if (m_RemovedList != null) {
            m_RemovedList.put(p_RemovedNode.getName(), p_RemovedNode);
        }
    }

    /**
     * Search the added node in the removed list and if it's found, copy the expanded info.
     *
     * @param p_AddedNode The added node
     */
    protected void autoRefresh(TreeControlNode p_AddedNode) {
        if (m_RemovedList != null) {
            TreeControlNode oRemove = (TreeControlNode) m_RemovedList.get(p_AddedNode.getName());
            if (oRemove != null) {
                p_AddedNode.setExpanded(oRemove.isExpanded());
            }
        }
    }
}
